@@ -213,8 +213,8 @@ jack_process_cb (jack_nframes_t nframes, void *arg)
     if (nframes * sizeof (sample_t) != flen)
       goto wrong_size;
 
-    GST_DEBUG ("copy %d frames: %p, %d bytes, %d channels", nframes, readptr,
-        flen, channels);
+    GST_DEBUG_OBJECT (sink, "copy %d frames: %p, %d bytes, %d channels",
+        nframes, readptr, flen, channels);
     data = (sample_t *) readptr;
 
     /* the samples in the ringbuffer have the channels interleaved, we need to
@@ -231,6 +231,7 @@ jack_process_cb (jack_nframes_t nframes, void *arg)
     /* we wrote one segment */
     gst_ring_buffer_advance (buf, 1);
   } else {
+    GST_DEBUG_OBJECT (sink, "write %d frames silence", nframes);
     /* We are not allowed to read from the ringbuffer, write silence to all
      * jack output buffers */
     for (i = 0; i < channels; i++) {
@@ -441,9 +442,17 @@ gst_jack_ring_buffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
       (GST_SECOND / GST_USECOND), spec->rate * spec->bytes_per_sample);
   /* segtotal based on buffer-time latency */
   spec->segtotal = spec->buffer_time / spec->latency_time;
+  if (spec->segtotal < 2) {
+    spec->segtotal = 2;
+    spec->buffer_time = spec->latency_time * spec->segtotal;
+  }
 
-  GST_DEBUG_OBJECT (sink, "segsize %d, segtotal %d", spec->segsize,
-      spec->segtotal);
+  GST_DEBUG_OBJECT (sink, "buffer time: %" G_GINT64_FORMAT " usec",
+      spec->buffer_time);
+  GST_DEBUG_OBJECT (sink, "latency time: %" G_GINT64_FORMAT " usec",
+      spec->latency_time);
+  GST_DEBUG_OBJECT (sink, "buffer_size %d, segsize %d, segtotal %d",
+      buffer_size, spec->segsize, spec->segtotal);
 
   /* allocate the ringbuffer memory now */
   buf->data = gst_buffer_new_and_alloc (spec->segtotal * spec->segsize);
@@ -607,7 +616,7 @@ gst_jack_ring_buffer_delay (GstRingBuffer * buf)
       res = latency;
   }
 
-  GST_DEBUG_OBJECT (sink, "delay %u", res);
+  GST_LOG_OBJECT (sink, "delay %u", res);
 
   return res;
 }
