@@ -45,7 +45,10 @@
 #include "gstlv2.h"
 #include <slv2/slv2.h>
 
-#define GST_SLV2_PLUGIN_QDATA g_quark_from_static_string("slv2-plugin")
+#define GST_LV2_DEFAULT_PATH \
+  "/usr/lib/lv2" G_SEARCHPATH_SEPARATOR_S \
+  "/usr/local/lib/lv2" G_SEARCHPATH_SEPARATOR_S \
+  LIBDIR "/lv2"
 
 static void gst_lv2_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -53,7 +56,7 @@ static void gst_lv2_set_property (GObject * object,
 static void gst_lv2_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
-static gboolean gst_lv2_setup (GstSignalProcessor * sigproc, guint sample_rate);
+static gboolean gst_lv2_setup (GstSignalProcessor * sigproc, GstCaps * caps);
 static gboolean gst_lv2_start (GstSignalProcessor * sigproc);
 static void gst_lv2_stop (GstSignalProcessor * sigproc);
 static void gst_lv2_cleanup (GstSignalProcessor * sigproc);
@@ -90,7 +93,10 @@ static GstPlugin *gst_lv2_plugin;
 GST_DEBUG_CATEGORY_STATIC (lv2_debug);
 #define GST_CAT_DEFAULT lv2_debug
 
-/** Convert an LV2 port role to a Gst channel positon
+static GQuark descriptor_quark = 0;
+
+
+/* Convert an LV2 port role to a Gst channel positon
  * WARNING: If the group has only a single port,
  * GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER will be returned for pg:centerRole
  * (which is used by LV2 for mono groups), but this is not correct.  In this
@@ -139,7 +145,7 @@ gst_lv2_role_to_position (SLV2Value role)
   return GST_AUDIO_CHANNEL_POSITION_INVALID;
 }
 
-/** Find and return the group @a uri in @a groups, or NULL if not found */
+/* Find and return the group @a uri in @a groups, or NULL if not found */
 static GstLV2Group *
 gst_lv2_class_find_group (GArray * groups, SLV2Value uri)
 {
@@ -153,16 +159,15 @@ gst_lv2_class_find_group (GArray * groups, SLV2Value uri)
 static GstAudioChannelPosition *
 gst_lv2_build_positions (GstLV2Group * group)
 {
-  int i;
   GstAudioChannelPosition *positions = NULL;
-  if (group->has_roles) {
-    positions = malloc (group->ports->len * sizeof (GstAudioChannelPosition));
+
+  /* don't do anything for mono */
+  if (group->ports->len > 1) {
+    gint i;
+
+    positions = g_new (GstAudioChannelPosition, group->ports->len);
     for (i = 0; i < group->ports->len; ++i)
       positions[i] = g_array_index (group->ports, GstLV2Port, i).position;
-
-    // Fix up mono groups (see WARNING above)
-    if (group->ports->len == 1)
-      positions[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
   }
   return positions;
 }
@@ -179,18 +184,17 @@ gst_lv2_base_init (gpointer g_class)
   SLV2Values values, sub_values;
   GstLV2Group *group = NULL;
   GstAudioChannelPosition position = GST_AUDIO_CHANNEL_POSITION_INVALID;
-  GstAudioChannelPosition mono_position = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
-  GstAudioChannelPosition *positions = NULL;
   guint j, in_pad_index = 0, out_pad_index = 0;
   gchar *klass_tags;
 
-  GST_DEBUG ("base_init %p", g_class);
-
   lv2plugin = (SLV2Plugin) g_type_get_qdata (G_OBJECT_CLASS_TYPE (klass),
-      GST_SLV2_PLUGIN_QDATA);
+      descriptor_quark);
 
   g_assert (lv2plugin);
 
+  GST_DEBUG ("base_init %p, plugin %s", g_class,
+      slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)));
+
   gsp_class->num_group_in = 0;
   gsp_class->num_group_out = 0;
   gsp_class->num_audio_in = 0;
@@ -210,7 +214,7 @@ gst_lv2_base_init (gpointer g_class)
     const SLV2Port port = slv2_plugin_get_port_by_index (lv2plugin, j);
     const gboolean is_input = slv2_port_is_a (lv2plugin, port, input_class);
     gboolean in_group = FALSE;
-    struct _GstLV2Port desc = { j, 0 };
+    struct _GstLV2Port desc = { j, 0, };
     values = slv2_port_get_value (lv2plugin, port, in_group_pred);
 
     if (slv2_values_size (values) > 0) {
@@ -238,13 +242,13 @@ gst_lv2_base_init (gpointer g_class)
             assert (group);
             assert (slv2_value_equals (group->uri, group_uri));
           } else {
-            GST_WARNING ("plugin %s has duplicate group symbol '%s'\n",
+            GST_WARNING ("plugin %s has duplicate group symbol '%s'",
                 slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)),
                 slv2_value_as_string (g.symbol));
             in_group = FALSE;
           }
         } else {
-          GST_WARNING ("plugin %s has illegal group with no symbol\n",
+          GST_WARNING ("plugin %s has illegal group with no symbol",
               slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)));
           in_group = FALSE;
         }
@@ -283,6 +287,7 @@ gst_lv2_base_init (gpointer g_class)
           g_array_append_val (klass->control_out_ports, desc);
       } else {
         /* unknown port type */
+        GST_INFO ("unhandled port %d", j);
         continue;
       }
     }
@@ -299,35 +304,19 @@ gst_lv2_base_init (gpointer g_class)
   /* add input group pad templates */
   for (j = 0; j < gsp_class->num_group_in; ++j) {
     group = &g_array_index (klass->in_groups, GstLV2Group, j);
-    if (group->has_roles) {
-      positions = gst_lv2_build_positions (group);
-    }
 
     gst_signal_processor_class_add_pad_template (gsp_class,
-        slv2_value_as_string (group->symbol),
-        GST_PAD_SINK, j, group->ports->len, positions);
-
-    if (group->has_roles) {
-      free (positions);
-      positions = NULL;
-    }
+        slv2_value_as_string (group->symbol), GST_PAD_SINK, j,
+        group->ports->len);
   }
 
   /* add output group pad templates */
   for (j = 0; j < gsp_class->num_group_out; ++j) {
     group = &g_array_index (klass->out_groups, GstLV2Group, j);
-    if (group->has_roles) {
-      positions = gst_lv2_build_positions (group);
-    }
 
     gst_signal_processor_class_add_pad_template (gsp_class,
-        slv2_value_as_string (group->symbol),
-        GST_PAD_SRC, j, group->ports->len, positions);
-
-    if (group->has_roles) {
-      free (positions);
-      positions = NULL;
-    }
+        slv2_value_as_string (group->symbol), GST_PAD_SRC, j,
+        group->ports->len);
   }
 
   /* add non-grouped input port pad templates */
@@ -338,7 +327,7 @@ gst_lv2_base_init (gpointer g_class)
     const gchar *name =
         slv2_value_as_string (slv2_port_get_symbol (lv2plugin, port));
     gst_signal_processor_class_add_pad_template (gsp_class, name, GST_PAD_SINK,
-        j, 1, &mono_position);
+        j, 1);
   }
 
   /* add non-grouped output port pad templates */
@@ -348,8 +337,8 @@ gst_lv2_base_init (gpointer g_class)
     SLV2Port port = slv2_plugin_get_port_by_index (lv2plugin, desc->index);
     const gchar *name =
         slv2_value_as_string (slv2_port_get_symbol (lv2plugin, port));
-    gst_signal_processor_class_add_pad_template (gsp_class, name, GST_PAD_SINK,
-        j, 1, &mono_position);
+    gst_signal_processor_class_add_pad_template (gsp_class, name, GST_PAD_SRC,
+        j, 1);
   }
 
   /* construct the element details struct */
@@ -394,14 +383,22 @@ gst_lv2_base_init (gpointer g_class)
 }
 
 static gchar *
-gst_lv2_class_get_param_name (GstLV2Class * klass, gint portnum)
+gst_lv2_class_get_param_name (GstLV2Class * klass, SLV2Port port)
 {
   SLV2Plugin lv2plugin = klass->plugin;
-  SLV2Port port = slv2_plugin_get_port_by_index (lv2plugin, portnum);
+
   return g_strdup (slv2_value_as_string (slv2_port_get_symbol (lv2plugin,
               port)));
 }
 
+static gchar *
+gst_lv2_class_get_param_nick (GstLV2Class * klass, SLV2Port port)
+{
+  SLV2Plugin lv2plugin = klass->plugin;
+
+  return g_strdup (slv2_value_as_string (slv2_port_get_name (lv2plugin, port)));
+}
+
 static GParamSpec *
 gst_lv2_class_get_param_spec (GstLV2Class * klass, gint portnum)
 {
@@ -409,11 +406,24 @@ gst_lv2_class_get_param_spec (GstLV2Class * klass, gint portnum)
   SLV2Port port = slv2_plugin_get_port_by_index (lv2plugin, portnum);
   SLV2Value lv2def, lv2min, lv2max;
   GParamSpec *ret;
-  gchar *name;
+  gchar *name, *nick;
   gint perms;
   gfloat lower = 0.0f, upper = 1.0f, def = 0.0f;
 
-  name = gst_lv2_class_get_param_name (klass, portnum);
+  nick = gst_lv2_class_get_param_nick (klass, port);
+  name = gst_lv2_class_get_param_name (klass, port);
+  g_strcanon (name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-", '-');
+  if (!((name[0] >= 'a' && name[0] <= 'z') || (name[0] >= 'A'
+              && name[0] <= 'Z'))) {
+    gchar *tempstr = name;
+
+    name = g_strconcat ("param-", name, NULL);
+    g_free (tempstr);
+  }
+
+  GST_DEBUG ("%s trying port %s : %s",
+      slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)), name, nick);
+
   perms = G_PARAM_READABLE;
   if (slv2_port_is_a (lv2plugin, port, input_class))
     perms |= G_PARAM_WRITABLE | G_PARAM_CONSTRUCT;
@@ -421,9 +431,8 @@ gst_lv2_class_get_param_spec (GstLV2Class * klass, gint portnum)
     perms |= GST_PARAM_CONTROLLABLE;
 
   if (slv2_port_has_property (lv2plugin, port, toggled_prop)) {
-    ret = g_param_spec_boolean (name, name, name, FALSE, perms);
-    g_free (name);
-    return ret;
+    ret = g_param_spec_boolean (name, nick, nick, FALSE, perms);
+    goto done;
   }
 
   slv2_port_get_range (lv2plugin, port, &lv2def, &lv2min, &lv2max);
@@ -440,23 +449,25 @@ gst_lv2_class_get_param_spec (GstLV2Class * klass, gint portnum)
   slv2_value_free (lv2max);
 
   if (def < lower) {
-    GST_WARNING ("%s has lower bound %f > default %f\n",
+    GST_WARNING ("%s has lower bound %f > default %f",
         slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)), lower, def);
     lower = def;
   }
 
   if (def > upper) {
-    GST_WARNING ("%s has upper bound %f < default %f\n",
+    GST_WARNING ("%s has upper bound %f < default %f",
         slv2_value_as_string (slv2_plugin_get_uri (lv2plugin)), upper, def);
     upper = def;
   }
 
   if (slv2_port_has_property (lv2plugin, port, integer_prop))
-    ret = g_param_spec_int (name, name, name, lower, upper, def, perms);
+    ret = g_param_spec_int (name, nick, nick, lower, upper, def, perms);
   else
-    ret = g_param_spec_float (name, name, name, lower, upper, def, perms);
+    ret = g_param_spec_float (name, nick, nick, lower, upper, def, perms);
 
+done:
   g_free (name);
+  g_free (nick);
 
   return ret;
 }
@@ -466,7 +477,8 @@ gst_lv2_class_init (GstLV2Class * klass, SLV2Plugin lv2plugin)
 {
   GObjectClass *gobject_class;
   GstSignalProcessorClass *gsp_class;
-  gint i;
+  GParamSpec *p;
+  gint i, ix;
 
   GST_DEBUG ("class_init %p", klass);
 
@@ -483,27 +495,23 @@ gst_lv2_class_init (GstLV2Class * klass, SLV2Plugin lv2plugin)
 
   klass->plugin = lv2plugin;
 
+  /* properties have an offset of 1 */
+  ix = 1;
+
   /* register properties */
 
-  for (i = 0; i < gsp_class->num_control_in; i++) {
-    GParamSpec *p;
-
+  for (i = 0; i < gsp_class->num_control_in; i++, ix++) {
     p = gst_lv2_class_get_param_spec (klass,
         g_array_index (klass->control_in_ports, GstLV2Port, i).index);
 
-    /* properties have an offset of 1 */
-    g_object_class_install_property (G_OBJECT_CLASS (klass), i + 1, p);
+    g_object_class_install_property (gobject_class, ix, p);
   }
 
-  for (i = 0; i < gsp_class->num_control_out; i++) {
-    GParamSpec *p;
-
+  for (i = 0; i < gsp_class->num_control_out; i++, ix++) {
     p = gst_lv2_class_get_param_spec (klass,
         g_array_index (klass->control_out_ports, GstLV2Port, i).index);
 
-    /* properties have an offset of 1, and we already added num_control_in */
-    g_object_class_install_property (G_OBJECT_CLASS (klass),
-        gsp_class->num_control_in + i + 1, p);
+    g_object_class_install_property (gobject_class, ix, p);
   }
 }
 
@@ -587,12 +595,17 @@ gst_lv2_get_property (GObject * object, guint prop_id, GValue * value,
 }
 
 static gboolean
-gst_lv2_setup (GstSignalProcessor * gsp, guint sample_rate)
+gst_lv2_setup (GstSignalProcessor * gsp, GstCaps * caps)
 {
   GstLV2 *lv2;
   GstLV2Class *oclass;
   GstSignalProcessorClass *gsp_class;
+  GstStructure *s;
   gint i;
+  GstLV2Group *group = NULL;
+  GstAudioChannelPosition *positions = NULL;
+  GstPad *pad;
+  GstCaps *pad_caps;
 
   gsp_class = GST_SIGNAL_PROCESSOR_GET_CLASS (gsp);
   lv2 = (GstLV2 *) gsp;
@@ -600,11 +613,11 @@ gst_lv2_setup (GstSignalProcessor * gsp, guint sample_rate)
 
   g_return_val_if_fail (lv2->activated == FALSE, FALSE);
 
-  GST_DEBUG_OBJECT (lv2, "instantiating the plugin at %d Hz", sample_rate);
+  GST_DEBUG_OBJECT (lv2, "instantiating the plugin at %d Hz", gsp->sample_rate);
 
-  lv2->instance = slv2_plugin_instantiate (oclass->plugin, sample_rate, NULL);
-
-  g_return_val_if_fail (lv2->instance != NULL, FALSE);
+  if (!(lv2->instance =
+          slv2_plugin_instantiate (oclass->plugin, gsp->sample_rate, NULL)))
+    goto no_instance;
 
   /* connect the control ports */
   for (i = 0; i < gsp_class->num_control_in; i++)
@@ -616,7 +629,51 @@ gst_lv2_setup (GstSignalProcessor * gsp, guint sample_rate)
         g_array_index (oclass->control_out_ports, GstLV2Port, i).index,
         &(gsp->control_out[i]));
 
+  /* set input group pad audio channel position */
+  for (i = 0; i < gsp_class->num_group_in; ++i) {
+    group = &g_array_index (oclass->in_groups, GstLV2Group, i);
+    if (group->has_roles) {
+      if ((positions = gst_lv2_build_positions (group))) {
+        if ((pad = gst_element_get_static_pad (GST_ELEMENT (gsp),
+                    slv2_value_as_string (group->symbol)))) {
+          GST_INFO_OBJECT (lv2, "set audio channel positions on sink pad %s",
+              slv2_value_as_string (group->symbol));
+          pad_caps = GST_PAD_CAPS (pad);
+          s = gst_caps_get_structure (caps, 0);
+          gst_audio_set_channel_positions (s, positions);
+          gst_object_unref (pad);
+        }
+        g_free (positions);
+        positions = NULL;
+      }
+    }
+  }
+  /* set output group pad audio channel position */
+  for (i = 0; i < gsp_class->num_group_out; ++i) {
+    group = &g_array_index (oclass->out_groups, GstLV2Group, i);
+    if (group->has_roles) {
+      if ((positions = gst_lv2_build_positions (group))) {
+        if ((pad = gst_element_get_static_pad (GST_ELEMENT (gsp),
+                    slv2_value_as_string (group->symbol)))) {
+          GST_INFO_OBJECT (lv2, "set audio channel positions on src pad %s",
+              slv2_value_as_string (group->symbol));
+          pad_caps = GST_PAD_CAPS (pad);
+          s = gst_caps_get_structure (caps, 0);
+          gst_audio_set_channel_positions (s, positions);
+          gst_object_unref (pad);
+        }
+        g_free (positions);
+        positions = NULL;
+      }
+    }
+  }
   return TRUE;
+
+no_instance:
+  {
+    GST_WARNING_OBJECT (gsp, "could not create instance");
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -671,7 +728,7 @@ gst_lv2_process (GstSignalProcessor * gsp, guint nframes)
 {
   GstSignalProcessorClass *gsp_class;
   GstLV2 *lv2;
-  GstLV2Class *oclass;
+  GstLV2Class *lv2_class;
   GstLV2Group *lv2_group;
   GstLV2Port *lv2_port;
   GstSignalProcessorGroup *gst_group;
@@ -679,10 +736,11 @@ gst_lv2_process (GstSignalProcessor * gsp, guint nframes)
 
   gsp_class = GST_SIGNAL_PROCESSOR_GET_CLASS (gsp);
   lv2 = (GstLV2 *) gsp;
-  oclass = (GstLV2Class *) gsp_class;
+  lv2_class = (GstLV2Class *) gsp_class;
 
+  /* multi channel inputs */
   for (i = 0; i < gsp_class->num_group_in; i++) {
-    lv2_group = &g_array_index (oclass->in_groups, GstLV2Group, i);
+    lv2_group = &g_array_index (lv2_class->in_groups, GstLV2Group, i);
     gst_group = &gsp->group_in[i];
     for (j = 0; j < lv2_group->ports->len; ++j) {
       lv2_port = &g_array_index (lv2_group->ports, GstLV2Port, j);
@@ -690,13 +748,15 @@ gst_lv2_process (GstSignalProcessor * gsp, guint nframes)
           gst_group->buffer + (j * nframes));
     }
   }
+  /* mono inputs */
   for (i = 0; i < gsp_class->num_audio_in; i++) {
-    lv2_port = &g_array_index (oclass->audio_in_ports, GstLV2Port, i);
+    lv2_port = &g_array_index (lv2_class->audio_in_ports, GstLV2Port, i);
     slv2_instance_connect_port (lv2->instance, lv2_port->index,
         gsp->audio_in[i]);
   }
+  /* multi channel outputs */
   for (i = 0; i < gsp_class->num_group_out; i++) {
-    lv2_group = &g_array_index (oclass->out_groups, GstLV2Group, i);
+    lv2_group = &g_array_index (lv2_class->out_groups, GstLV2Group, i);
     gst_group = &gsp->group_out[i];
     for (j = 0; j < lv2_group->ports->len; ++j) {
       lv2_port = &g_array_index (lv2_group->ports, GstLV2Port, j);
@@ -704,8 +764,9 @@ gst_lv2_process (GstSignalProcessor * gsp, guint nframes)
           gst_group->buffer + (j * nframes));
     }
   }
+  /* mono outputs */
   for (i = 0; i < gsp_class->num_audio_out; i++) {
-    lv2_port = &g_array_index (oclass->audio_out_ports, GstLV2Port, i);
+    lv2_port = &g_array_index (lv2_class->audio_out_ports, GstLV2Port, i);
     slv2_instance_connect_port (lv2->instance, lv2_port->index,
         gsp->audio_out[i]);
   }
@@ -718,10 +779,14 @@ gst_lv2_process (GstSignalProcessor * gsp, guint nframes)
 static gboolean
 lv2_plugin_discover (void)
 {
-  guint i;
+  guint i, j;
   SLV2Plugins plugins = slv2_world_get_all_plugins (world);
+
   for (i = 0; i < slv2_plugins_size (plugins); ++i) {
     SLV2Plugin lv2plugin = slv2_plugins_get_at (plugins, i);
+    gint num_audio_ports = 0;
+    const gchar *plugin_uri, *p;
+    gchar *type_name;
     GTypeInfo typeinfo = {
       sizeof (GstLV2Class),
       (GBaseInitFunc) gst_lv2_base_init,
@@ -733,18 +798,34 @@ lv2_plugin_discover (void)
       0,
       (GInstanceInitFunc) gst_lv2_init,
     };
-
     GType type;
 
+    plugin_uri = slv2_value_as_uri (slv2_plugin_get_uri (lv2plugin));
     /* construct the type name from plugin URI */
-    gchar *type_name = g_strdup_printf ("%s",
-        slv2_value_as_uri (slv2_plugin_get_uri (lv2plugin)));
+    if ((p = strstr (plugin_uri, "://"))) {
+      /* cut off the protocol (e.g. http://) */
+      type_name = g_strdup (&p[3]);
+    } else {
+      type_name = g_strdup (plugin_uri);
+    }
     g_strcanon (type_name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-+", '-');
 
     /* if it's already registered, drop it */
     if (g_type_from_name (type_name))
       goto next;
 
+    /* check if this has any audio ports */
+    for (j = 0; j < slv2_plugin_get_num_ports (lv2plugin); j++) {
+      const SLV2Port port = slv2_plugin_get_port_by_index (lv2plugin, j);
+      if (slv2_port_is_a (lv2plugin, port, audio_class)) {
+        num_audio_ports++;
+      }
+    }
+    if (!num_audio_ports) {
+      GST_INFO ("plugin %s has no audio ports", type_name);
+      goto next;
+    }
+
     /* create the type */
     type =
         g_type_register_static (GST_TYPE_SIGNAL_PROCESSOR, type_name, &typeinfo,
@@ -752,7 +833,7 @@ lv2_plugin_discover (void)
 
     /* FIXME: not needed anymore when we can add pad templates, etc in class_init
      * as class_data contains the LADSPA_Descriptor too */
-    g_type_set_qdata (type, GST_SLV2_PLUGIN_QDATA, (gpointer) lv2plugin);
+    g_type_set_qdata (type, descriptor_quark, (gpointer) lv2plugin);
 
     if (!gst_element_register (gst_lv2_plugin, type_name, GST_RANK_NONE, type))
       goto next;
@@ -799,9 +880,20 @@ plugin_init (GstPlugin * plugin)
   side_left_role = slv2_value_new_uri (world, NS_PG "sideLeftChannel");
   side_right_role = slv2_value_new_uri (world, NS_PG "sideRightChannel");
 
+  /* initialize gst controller library */
+  gst_controller_init (NULL, NULL);
+
+  gst_plugin_add_dependency_simple (plugin,
+      "LV2_PATH", GST_LV2_DEFAULT_PATH, NULL, GST_PLUGIN_DEPENDENCY_FLAG_NONE);
+
   parent_class = g_type_class_ref (GST_TYPE_SIGNAL_PROCESSOR);
 
   gst_lv2_plugin = plugin;
+  descriptor_quark = g_quark_from_static_string ("slv2-plugin");
+
+  /* ensure GstAudioChannelPosition type is registered */
+  if (!gst_audio_channel_position_get_type ())
+    return FALSE;
 
   return lv2_plugin_discover ();
 }
