@@ -51,6 +51,12 @@
 GST_DEBUG_CATEGORY_STATIC (gst_timidity_debug);
 #define GST_CAT_DEFAULT gst_timidity_debug
 
+static const GstElementDetails gst_timidity_details =
+GST_ELEMENT_DETAILS ("Timidity",
+    "Codec/Decoder/Audio",
+    "Midi Synthesizer Element",
+    "Wouter Paesen <wouter@blue-gate.be>");
+
 enum
 {
   /* FILL ME */
@@ -63,6 +69,9 @@ enum
   /* FILL ME */
 };
 
+static void gst_timidity_base_init (gpointer g_class);
+static void gst_timidity_class_init (GstTimidityClass * klass);
+
 static gboolean gst_timidity_src_event (GstPad * pad, GstEvent * event);
 static GstStateChangeReturn gst_timidity_change_state (GstElement * element,
     GstStateChange transition);
@@ -95,20 +104,23 @@ gst_timidity_base_init (gpointer gclass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
-  gst_element_class_add_static_pad_template (element_class, &sink_factory);
-  gst_element_class_set_details_simple (element_class, "Timidity",
-      "Codec/Decoder/Audio",
-      "Midi Synthesizer Element", "Wouter Paesen <wouter@blue-gate.be>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_set_details (element_class, &gst_timidity_details);
 }
 
 /* initialize the plugin's class */
 static void
 gst_timidity_class_init (GstTimidityClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
+
   gstelement_class->change_state = gst_timidity_change_state;
 }
 
@@ -123,7 +135,7 @@ gst_timidity_init (GstTimidity * filter, GstTimidityClass * g_class)
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (filter);
 
   /* initialise timidity library */
-  if (mid_init ((char *) TIMIDITY_CFG) == 0) {
+  if (mid_init (TIMIDITY_CFG) == 0) {
     filter->initialized = TRUE;
   } else {
     GST_WARNING ("can't initialize timidity with config: " TIMIDITY_CFG);
@@ -313,7 +325,7 @@ static GstSegment *
 gst_timidity_get_segment (GstTimidity * timidity, GstFormat format,
     gboolean update)
 {
-  gint64 start = 0, stop = 0, time = 0;
+  gint64 start, stop, time;
 
   GstSegment *segment = gst_segment_new ();
 
@@ -372,7 +384,7 @@ gst_timidity_src_event (GstPad * pad, GstEvent * event)
       GstFormat src_format, dst_format;
       GstSeekFlags flags;
       GstSeekType start_type, stop_type;
-      gint64 orig_start, start = 0, stop = 0;
+      gint64 orig_start, start, stop;
       gboolean flush, update;
 
       if (!timidity->song)
@@ -534,7 +546,7 @@ gst_timidity_fill_buffer (GstTimidity * timidity, GstBuffer * buffer)
 
   GST_DEBUG_OBJECT (timidity,
       "generated buffer %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT
-      " (%" G_GINT64_FORMAT " samples)",
+      " (%d samples)",
       GST_TIME_ARGS ((guint64) GST_BUFFER_TIMESTAMP (buffer)),
       GST_TIME_ARGS (((guint64) (GST_BUFFER_TIMESTAMP (buffer) +
                   GST_BUFFER_DURATION (buffer)))), samples);
@@ -709,9 +721,7 @@ gst_timidity_loop (GstPad * sinkpad)
   gst_buffer_set_caps (out, timidity->out_caps);
   ret = gst_pad_push (timidity->srcpad, out);
 
-  if (ret == GST_FLOW_UNEXPECTED)
-    goto eos;
-  else if (ret < GST_FLOW_UNEXPECTED || ret == GST_FLOW_NOT_LINKED)
+  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED)
     goto error;
 
   return;
@@ -722,11 +732,6 @@ paused:
     gst_pad_pause_task (timidity->sinkpad);
     return;
   }
-eos:
-  {
-    gst_pad_push_event (timidity->srcpad, gst_event_new_eos ());
-    goto paused;
-  }
 error:
   {
     GST_ELEMENT_ERROR (timidity, STREAM, FAILED,
