@@ -24,9 +24,7 @@
  * SECTION:element-jifmux
  * @short_description: JPEG interchange format writer
  *
- * Writes a JPEG image as JPEG/EXIF or JPEG/JFIF including various metadata. The
- * jpeg image received on the sink pad should be minimal (e.g. should not
- * contain metadata already).
+ * Writes a JPEG image as JPEG/EXIF or JPEG/JFIF including various metadata.
  *
  * <refsect2>
  * <title>Example launch line</title>
@@ -57,8 +55,6 @@ gst-launch videotestsrc num-buffers=1 ! jpegenc ! taginject tags="comment=\"test
 #include <string.h>
 #include <gst/base/gstbytereader.h>
 #include <gst/base/gstbytewriter.h>
-#include <gst/tag/tag.h>
-#include <gst/tag/xmpwriter.h>
 
 #include "gstjifmux.h"
 
@@ -79,13 +75,6 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 GST_DEBUG_CATEGORY_STATIC (jif_mux_debug);
 #define GST_CAT_DEFAULT jif_mux_debug
 
-#define COLORSPACE_UNKNOWN         (0 << 0)
-#define COLORSPACE_GRAYSCALE       (1 << 0)
-#define COLORSPACE_YUV             (1 << 1)
-#define COLORSPACE_RGB             (1 << 2)
-#define COLORSPACE_CMYK            (1 << 3)
-#define COLORSPACE_YCCK            (1 << 4)
-
 typedef struct _GstJifMuxMarker
 {
   guint8 marker;
@@ -101,29 +90,26 @@ struct _GstJifMuxPrivate
 
   /* list of GstJifMuxMarker */
   GList *markers;
-  guint scan_size;
+  guint16 scan_size;
   const guint8 *scan_data;
 };
 
+static void gst_jif_mux_base_init (gpointer g_class);
+static void gst_jif_mux_class_init (GstJifMuxClass * klass);
 static void gst_jif_mux_finalize (GObject * object);
 
 static void gst_jif_mux_reset (GstJifMux * self);
 static gboolean gst_jif_mux_sink_setcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_jif_mux_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_jif_mux_chain (GstPad * pad, GstBuffer * buffer);
-static GstStateChangeReturn gst_jif_mux_change_state (GstElement * element,
-    GstStateChange transition);
 
 
 static void
 gst_jif_type_init (GType type)
 {
   static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
-  static const GInterfaceInfo tag_xmp_writer_info = { NULL, NULL, NULL };
 
   g_type_add_interface_static (type, GST_TYPE_TAG_SETTER, &tag_setter_info);
-  g_type_add_interface_static (type, GST_TYPE_TAG_XMP_WRITER,
-      &tag_xmp_writer_info);
 
   GST_DEBUG_CATEGORY_INIT (jif_mux_debug, "jifmux", 0,
       "JPEG interchange format muxer");
@@ -137,14 +123,14 @@ gst_jif_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jif_mux_src_pad_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_jif_mux_sink_pad_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jif_mux_src_pad_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_jif_mux_sink_pad_template));
   gst_element_class_set_details_simple (element_class,
-      "JPEG stream muxer",
-      "Video/Formatter",
-      "Remuxes JPEG images with markers and tags",
+      "JPEG stream parser",
+      "Codec/Parser/Video",
+      "Parse JPEG images into single-frame buffers",
       "Arnout Vandecappelle (Essensium/Mind) <arnout@mind.be>");
 }
 
@@ -152,15 +138,11 @@ static void
 gst_jif_mux_class_init (GstJifMuxClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
 
   g_type_class_add_private (gobject_class, sizeof (GstJifMuxPrivate));
 
-  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_jif_mux_change_state);
-
   gobject_class->finalize = gst_jif_mux_finalize;
 }
 
@@ -199,17 +181,17 @@ gst_jif_mux_finalize (GObject * object)
 static gboolean
 gst_jif_mux_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
-  GstJifMux *self = GST_JIF_MUX_CAST (GST_PAD_PARENT (pad));
+  GstJifMux *self = GST_JIF_MUX (GST_PAD_PARENT (pad));
   GstStructure *s = gst_caps_get_structure (caps, 0);
   const gchar *variant;
 
   /* should be {combined (default), EXIF, JFIF} */
   if ((variant = gst_structure_get_string (s, "variant")) != NULL) {
-    GST_INFO_OBJECT (pad, "muxing to '%s'", variant);
+    GST_INFO_OBJECT (self, "muxing to '%s'", variant);
     /* FIXME: do we want to switch it like this or use a gobject property ? */
   }
 
-  return gst_pad_set_caps (self->priv->srcpad, caps);
+  return TRUE;
 }
 
 static gboolean
@@ -225,7 +207,6 @@ gst_jif_mux_sink_event (GstPad * pad, GstEvent * event)
       const GstTagMergeMode mode = gst_tag_setter_get_tag_merge_mode (setter);
 
       gst_event_parse_tag (event, &list);
-
       gst_tag_setter_merge_tags (setter, list, mode);
       break;
     }
@@ -236,15 +217,6 @@ gst_jif_mux_sink_event (GstPad * pad, GstEvent * event)
   return ret;
 }
 
-static void
-gst_jif_mux_marker_free (GstJifMuxMarker * m)
-{
-  if (m->owned)
-    g_free ((gpointer) m->data);
-
-  g_slice_free (GstJifMuxMarker, m);
-}
-
 static void
 gst_jif_mux_reset (GstJifMux * self)
 {
@@ -253,7 +225,11 @@ gst_jif_mux_reset (GstJifMux * self)
 
   for (node = self->priv->markers; node; node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
-    gst_jif_mux_marker_free (m);
+
+    if (m->owned)
+      g_free ((gpointer) m->data);
+
+    g_slice_free (GstJifMuxMarker, m);
   }
   g_list_free (self->priv->markers);
   self->priv->markers = NULL;
@@ -278,11 +254,9 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
 {
   GstByteReader reader = GST_BYTE_READER_INIT_FROM_BUFFER (buf);
   GstJifMuxMarker *m;
-  guint8 marker = 0;
-  guint16 size = 0;
-  const guint8 *data = NULL;
-
-  GST_LOG_OBJECT (self, "Received buffer of size: %u", GST_BUFFER_SIZE (buf));
+  guint8 marker;
+  guint16 size;
+  const guint8 *data;
 
   if (!gst_byte_reader_peek_uint8 (&reader, &marker))
     goto error;
@@ -319,7 +293,6 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
           goto error;
         if (!gst_byte_reader_get_data (&reader, size - 2, &data))
           goto error;
-
         m = gst_jif_mux_new_marker (marker, size - 2, data, FALSE);
         self->priv->markers = g_list_prepend (self->priv->markers, m);
 
@@ -328,26 +301,9 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
     }
 
     if (marker == SOS) {
-      gint eoi_pos = -1;
-      gint i;
-
-      /* search the last 5 bytes for the EOI marker */
-      g_assert (GST_BUFFER_SIZE (buf) >= 5);
-      for (i = 5; i >= 2; i--) {
-        if (GST_BUFFER_DATA (buf)[GST_BUFFER_SIZE (buf) - i] == 0xFF &&
-            GST_BUFFER_DATA (buf)[GST_BUFFER_SIZE (buf) - i + 1] == EOI) {
-          eoi_pos = GST_BUFFER_SIZE (buf) - i;
-          break;
-        }
-      }
-
-      if (eoi_pos == -1) {
-        GST_WARNING_OBJECT (self, "Couldn't find an EOI marker");
-        eoi_pos = GST_BUFFER_SIZE (buf);
-      }
-
       /* remaining size except EOI is scan data */
-      self->priv->scan_size = eoi_pos - gst_byte_reader_get_pos (&reader);
+      self->priv->scan_size = GST_BUFFER_SIZE (buf) - 4 -
+          gst_byte_reader_get_pos (&reader);
       if (!gst_byte_reader_get_data (&reader, self->priv->scan_size,
               &self->priv->scan_data))
         goto error;
@@ -376,21 +332,17 @@ static gboolean
 gst_jif_mux_mangle_markers (GstJifMux * self)
 {
   gboolean modified = FALSE;
-  GstTagList *tags = NULL;
-  gboolean cleanup_tags;
+  const GstTagList *tags;
   GstJifMuxMarker *m;
   GList *node, *file_hdr = NULL, *frame_hdr = NULL, *scan_hdr = NULL;
-  GList *app0_jfif = NULL, *app1_exif = NULL, *app1_xmp = NULL, *com = NULL;
-  GstBuffer *xmp_data;
-  gchar *str = NULL;
-  gint colorspace = COLORSPACE_UNKNOWN;
 
-  /* update the APP markers
-   * - put any JFIF APP0 first
-   * - the Exif APP1 next,
-   * - the XMP APP1 next,
-   * - the PSIR APP13 next,
-   * - followed by all other marker segments
+  /* FIXME: implement me more
+   * - update the APP markers
+   *   - put any JFIF APP0 first
+   *   - the Exif APP1 next, 
+   *   - the XMP APP1 next, 
+   *   - the PSIR APP13 next,
+   *   - followed by all other marker segments
    */
 
   /* find some reference points where we insert before/after */
@@ -399,64 +351,6 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
     m = (GstJifMuxMarker *) node->data;
 
     switch (m->marker) {
-      case APP0:
-        if (m->size > 5 && !memcmp (m->data, "JFIF\0", 5)) {
-          GST_DEBUG_OBJECT (self, "found APP0 JFIF");
-          colorspace |= COLORSPACE_GRAYSCALE | COLORSPACE_YUV;
-          if (!app0_jfif)
-            app0_jfif = node;
-        }
-        break;
-      case APP1:
-        if (m->size > 6 && (!memcmp (m->data, "EXIF\0\0", 6) ||
-                !memcmp (m->data, "Exif\0\0", 6))) {
-          GST_DEBUG_OBJECT (self, "found APP1 EXIF");
-          if (!app1_exif)
-            app1_exif = node;
-        } else if (m->size > 29
-            && !memcmp (m->data, "http://ns.adobe.com/xap/1.0/\0", 29)) {
-          GST_INFO_OBJECT (self, "found APP1 XMP, will be replaced");
-          if (!app1_xmp)
-            app1_xmp = node;
-        }
-        break;
-      case APP14:
-        /* check if this contains RGB */
-        /*
-         * This marker should have:
-         * - 'Adobe\0'
-         * - 2 bytes DCTEncodeVersion
-         * - 2 bytes flags0
-         * - 2 bytes flags1
-         * - 1 byte  ColorTransform
-         *             - 0 means unknown (RGB or CMYK)
-         *             - 1 YCbCr
-         *             - 2 YCCK
-         */
-
-        if ((m->size >= 14)
-            && (strncmp ((gchar *) m->data, "Adobe\0", 6) == 0)) {
-          switch (m->data[11]) {
-            case 0:
-              colorspace |= COLORSPACE_RGB | COLORSPACE_CMYK;
-              break;
-            case 1:
-              colorspace |= COLORSPACE_YUV;
-              break;
-            case 2:
-              colorspace |= COLORSPACE_YCCK;
-              break;
-            default:
-              break;
-          }
-        }
-
-        break;
-      case COM:
-        GST_INFO_OBJECT (self, "found COM, will be replaced");
-        if (!com)
-          com = node;
-        break;
       case DQT:
       case SOF0:
       case SOF1:
@@ -486,151 +380,37 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
 
   /* if we want combined or JFIF */
   /* check if we don't have JFIF APP0 */
-  if (!app0_jfif && (colorspace & (COLORSPACE_GRAYSCALE | COLORSPACE_YUV))) {
-    /* build jfif header */
-    static const struct
-    {
-      gchar id[5];
-      guint8 ver[2];
-      guint8 du;
-      guint8 xd[2], yd[2];
-      guint8 tw, th;
-    } jfif_data = {
-      "JFIF", {
-      1, 2}, 0, {
-      0, 1},                    /* FIXME: check pixel-aspect from caps */
-      {
-    0, 1}, 0, 0};
-    m = gst_jif_mux_new_marker (APP0, sizeof (jfif_data),
-        (const guint8 *) &jfif_data, FALSE);
-    /* insert into self->markers list */
-    self->priv->markers = g_list_insert (self->priv->markers, m, 1);
-    app0_jfif = g_list_nth (self->priv->markers, 1);
-  }
+  /* insert into self->markers list */
+  /* ensure its first */
   /* else */
   /* remove JFIF if exists */
 
-  /* Existing exif tags will be removed and our own will be added */
-  if (!tags) {
-    tags = (GstTagList *) gst_tag_setter_get_tag_list (GST_TAG_SETTER (self));
-    cleanup_tags = FALSE;
-  }
-  if (!tags) {
-    tags = gst_tag_list_new ();
-    cleanup_tags = TRUE;
-  }
+  /* if we want combined or EXIF */
+  /* check if we don't have EXIF APP1 */
+  /* insert into self->markers list */
+  /* else */
+  /* remove EXIF if exists */
 
-  GST_DEBUG_OBJECT (self, "Tags to be serialized %" GST_PTR_FORMAT, tags);
+  /* add jpeg comment */
+  tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (self));
+  if (tags) {
+    gchar *str = NULL;
 
-  /* FIXME: not happy with those
-   * - else where we would use VIDEO_CODEC = "Jpeg"
-   gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE,
-   GST_TAG_VIDEO_CODEC, "image/jpeg", NULL);
-   */
+    (void) (gst_tag_list_get_string (tags, GST_TAG_COMMENT, &str) ||
+        gst_tag_list_get_string (tags, GST_TAG_DESCRIPTION, &str) ||
+        gst_tag_list_get_string (tags, GST_TAG_TITLE, &str));
 
-  /* Add EXIF */
-  {
-    GstBuffer *exif_data;
-    guint8 *data;
-    GstJifMuxMarker *m;
-    GList *pos;
-
-    /* insert into self->markers list */
-    exif_data = gst_tag_list_to_exif_buffer_with_tiff_header (tags);
-    if (exif_data &&
-        GST_BUFFER_SIZE (exif_data) + 8 >= G_GUINT64_CONSTANT (65536)) {
-      GST_WARNING_OBJECT (self, "Exif tags data size exceed maximum size");
-      gst_buffer_unref (exif_data);
-      exif_data = NULL;
-    }
-    if (exif_data) {
-      data = g_malloc0 (GST_BUFFER_SIZE (exif_data) + 6);
-      memcpy (data, "Exif", 4);
-      memcpy (data + 6, GST_BUFFER_DATA (exif_data),
-          GST_BUFFER_SIZE (exif_data));
-      m = gst_jif_mux_new_marker (APP1, GST_BUFFER_SIZE (exif_data) + 6, data,
+    if (str) {
+      /* insert new marker into self->markers list */
+      m = gst_jif_mux_new_marker (COM, strlen (str) + 1, (const guint8 *) str,
           TRUE);
-      gst_buffer_unref (exif_data);
+      /* this should go before SOS, maybe at the end of file-header */
+      self->priv->markers = g_list_insert_before (self->priv->markers,
+          frame_hdr, m);
 
-      if (app1_exif) {
-        gst_jif_mux_marker_free ((GstJifMuxMarker *) app1_exif->data);
-        app1_exif->data = m;
-      } else {
-        pos = file_hdr;
-        if (app0_jfif)
-          pos = app0_jfif;
-        pos = g_list_next (pos);
-
-        self->priv->markers =
-            g_list_insert_before (self->priv->markers, pos, m);
-        if (pos) {
-          app1_exif = g_list_previous (pos);
-        } else {
-          app1_exif = g_list_last (self->priv->markers);
-        }
-      }
       modified = TRUE;
     }
   }
-
-  /* add xmp */
-  xmp_data =
-      gst_tag_xmp_writer_tag_list_to_xmp_buffer (GST_TAG_XMP_WRITER (self),
-      tags, FALSE);
-  if (xmp_data) {
-    guint8 *data, *xmp = GST_BUFFER_DATA (xmp_data);
-    guint size = GST_BUFFER_SIZE (xmp_data);
-    GList *pos;
-
-    data = g_malloc (size + 29);
-    memcpy (data, "http://ns.adobe.com/xap/1.0/\0", 29);
-    memcpy (&data[29], xmp, size);
-    m = gst_jif_mux_new_marker (APP1, size + 29, data, TRUE);
-
-    /*
-     * Replace the old xmp marker and not add a new one.
-     * There shouldn't be a xmp packet in the input, but it is better
-     * to be safe than add another one and end up with 2 packets.
-     */
-    if (app1_xmp) {
-      gst_jif_mux_marker_free ((GstJifMuxMarker *) app1_xmp->data);
-      app1_xmp->data = m;
-    } else {
-
-      pos = file_hdr;
-      if (app1_exif)
-        pos = app1_exif;
-      else if (app0_jfif)
-        pos = app0_jfif;
-      pos = g_list_next (pos);
-
-      self->priv->markers = g_list_insert_before (self->priv->markers, pos, m);
-
-    }
-    gst_buffer_unref (xmp_data);
-    modified = TRUE;
-  }
-
-  /* add jpeg comment from any of those */
-  (void) (gst_tag_list_get_string (tags, GST_TAG_COMMENT, &str) ||
-      gst_tag_list_get_string (tags, GST_TAG_DESCRIPTION, &str) ||
-      gst_tag_list_get_string (tags, GST_TAG_TITLE, &str));
-
-  if (str) {
-    GST_DEBUG_OBJECT (self, "set COM marker to '%s'", str);
-    /* insert new marker into self->markers list */
-    m = gst_jif_mux_new_marker (COM, strlen (str) + 1, (const guint8 *) str,
-        TRUE);
-    /* FIXME: if we have one already, replace */
-    /* this should go before SOS, maybe at the end of file-header */
-    self->priv->markers = g_list_insert_before (self->priv->markers,
-        frame_hdr, m);
-
-    modified = TRUE;
-  }
-
-  if (tags && cleanup_tags)
-    gst_tag_list_free (tags);
   return modified;
 }
 
@@ -644,7 +424,6 @@ gst_jif_mux_recombine_image (GstJifMux * self, GstBuffer ** new_buf,
   GstJifMuxMarker *m;
   GList *node;
   guint size = self->priv->scan_size;
-  gboolean writer_status = TRUE;
 
   /* iterate list and collect size */
   for (node = self->priv->markers; node; node = g_list_next (node)) {
@@ -673,35 +452,27 @@ gst_jif_mux_recombine_image (GstJifMux * self, GstBuffer ** new_buf,
 
   /* memcopy markers */
   writer = gst_byte_writer_new_with_buffer (buf, TRUE);
-  for (node = self->priv->markers; node && writer_status;
-      node = g_list_next (node)) {
+  for (node = self->priv->markers; node; node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
 
-    writer_status &= gst_byte_writer_put_uint8 (writer, 0xff);
-    writer_status &= gst_byte_writer_put_uint8 (writer, m->marker);
+    gst_byte_writer_put_uint8 (writer, 0xff);
+    gst_byte_writer_put_uint8 (writer, m->marker);
 
     GST_DEBUG_OBJECT (self, "marker = %2x, size = %u", m->marker, m->size + 2);
 
     if (m->size) {
-      writer_status &= gst_byte_writer_put_uint16_be (writer, m->size + 2);
-      writer_status &= gst_byte_writer_put_data (writer, m->data, m->size);
+      gst_byte_writer_put_uint16_be (writer, m->size + 2);
+      gst_byte_writer_put_data (writer, m->data, m->size);
     }
 
     if (m->marker == SOS) {
       GST_DEBUG_OBJECT (self, "scan data, size = %u", self->priv->scan_size);
-      writer_status &=
-          gst_byte_writer_put_data (writer, self->priv->scan_data,
+      gst_byte_writer_put_data (writer, self->priv->scan_data,
           self->priv->scan_size);
     }
   }
   gst_byte_writer_free (writer);
 
-  if (!writer_status) {
-    GST_WARNING_OBJECT (self, "Failed to write to buffer, calculated size "
-        "was probably too short");
-    g_assert_not_reached ();
-  }
-
   *new_buf = buf;
   return GST_FLOW_OK;
 
@@ -715,17 +486,11 @@ static GstFlowReturn
 gst_jif_mux_chain (GstPad * pad, GstBuffer * buf)
 {
   GstJifMux *self = GST_JIF_MUX (GST_PAD_PARENT (pad));
+  guint8 *data = GST_BUFFER_DATA (buf);
   GstFlowReturn fret = GST_FLOW_OK;
 
-  if (GST_BUFFER_CAPS (buf) == NULL) {
-    GST_WARNING_OBJECT (self, "Rejecting buffer without caps");
-    gst_buffer_unref (buf);
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
-
-  GST_MEMDUMP ("jpeg beg", GST_BUFFER_DATA (buf), 64);
-  GST_MEMDUMP ("jpeg end", GST_BUFFER_DATA (buf) + GST_BUFFER_SIZE (buf) - 64,
-      64);
+  GST_MEMDUMP ("jpeg beg", data, 64);
+  GST_MEMDUMP ("jpeg end", &data[GST_BUFFER_SIZE (buf) - 64], 64);
 
   /* we should have received a whole picture from SOI to EOI
    * build a list of markers */
@@ -747,37 +512,3 @@ gst_jif_mux_chain (GstPad * pad, GstBuffer * buf)
   }
   return fret;
 }
-
-static GstStateChangeReturn
-gst_jif_mux_change_state (GstElement * element, GstStateChange transition)
-{
-  GstStateChangeReturn ret;
-  GstJifMux *self = GST_JIF_MUX_CAST (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      break;
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_tag_setter_reset_tags (GST_TAG_SETTER (self));
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-
-  return ret;
-}
