@@ -1458,32 +1458,16 @@ mxf_primer_pack_to_buffer (const MXFPrimerPack * pack)
   if (pack->mappings) {
     guint local_tag;
     MXFUL *ul;
-#if GLIB_CHECK_VERSION (2, 16, 0)
     GHashTableIter iter;
 
     g_hash_table_iter_init (&iter, pack->mappings);
-#else
-    GList *l, *keys;
 
-    keys = g_hash_table_get_keys (pack->mappings);
-#endif
-
-#if GLIB_CHECK_VERSION (2, 16, 0)
     while (g_hash_table_iter_next (&iter, (gpointer) & local_tag,
             (gpointer) & ul)) {
-#else
-    for (l = keys; l; l = l->next) {
-      local_tag = GPOINTER_TO_UINT (l->data);
-      ul = g_hash_table_lookup (pack->mappings, GUINT_TO_POINTER (local_tag));
-#endif
       GST_WRITE_UINT16_BE (data, local_tag);
       memcpy (data + 2, ul, 16);
       data += 18;
     }
-
-#if !GLIB_CHECK_VERSION (2, 16, 0)
-    g_list_free (keys);
-#endif
   }
 
   return ret;
@@ -1553,10 +1537,11 @@ mxf_local_tag_add_to_hash_table (const MXFPrimerPack * primer,
     GST_DEBUG ("Adding local tag 0x%04x with UL %s and size %u", tag,
         mxf_ul_to_string (ul, str), tag_size);
 
-    local_tag = g_slice_new (MXFLocalTag);
+    local_tag = g_slice_new0 (MXFLocalTag);
     memcpy (&local_tag->ul, ul, sizeof (MXFUL));
     local_tag->size = tag_size;
     local_tag->data = g_memdup (tag_data, tag_size);
+    local_tag->g_slice = FALSE;
 
     g_hash_table_insert (*hash_table, &local_tag->ul, local_tag);
   } else {
